var MenuBuilder = function (restEngine, ZHPromise, templateManager, metaDataFactory, eventAggregator, $window, requestDataSourceEngine, ApplicationContext) {
    var localProductMenu;
    var attrToReplace = [
        "label",
        "showMenu",
        "showHeader",
        "showDescriptionOrder",
        "onRequestIdleTimeout",
        "mainWindow",
        "logLevel",
        "aclEnable",
        "showMenuSearch",
        "userDataActions",
        "reportServiceUrl",
        "reportsPath",
        "inDeveloperMode",
        "showDevtoolsInSameWindow",
        "shouldBackToMainWindowOnReload",
        "shortcutKeys"
    ];

    var requestProductMenu = function () {
        var modulesPackage = $window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/containers.json").then(function (response) {
            var productObjMenu = response.data;
            var keys = Object.keys(productObjMenu);
            // get first project menu found
            return productObjMenu[keys[0]];
        });
    };

    var removeEmptyMenuGroups = function(groupMenus) {
        return groupMenus.filter(function(group) {
            return (group.menus && group.menus.length != 0) || group.name == "zhHistory";
        });
    };

    var orderMenus = function(menus) {
        menus.map(function(menu) {
            if(menu.menus) {
                menu.menus = orderMenus(menu.menus);
            }
            return menu;
        });

        return sortMenus(menus);
    };

    var sortMenus = function(menus) {
        return menus.sort(function(menuA, menuB) {
            if (menuA.order && menuB.order) {
                if (menuA.order < menuB.order) {
                    return -1;
                }
                if (menuA.order > menuB.order) {
                    return 1;
                }
            }
            if (!menuA.order && menuB.order) {
                return -1;
            }
            if (menuA.order && menuB.order) {
                return 1;
            }
            return 0;
        });
    };

    var prepareMenus = function (response) {
        var groupMenus = response.dataset.menu;
        window.routesTranslate = response.dataset.routesTranslate;
        var zhuLogWidget = templateManager.container ? templateManager.container.getWidget('zhuLog') : false;
        if(zhuLogWidget) {
            zhuLogWidget.getField('ROUTE_LABEL').dataSource.data = window.routesTranslate;
            $(window).trigger('resize.grid');
        }
        groupMenus = orderMenus(removeEmptyMenuGroups(groupMenus));
        var groupMenuHistory = [templateManager.project.getMenu("zhHistory")];
        templateManager.project.groupMenu = metaDataFactory.groupMenuFactory(groupMenus.concat(groupMenuHistory));

        //Force framework to apply ACL Rules
        var aux = {};
        Object.keys(templateManager.aclRules).forEach(function(ruleKey){
            aux[ruleKey] = templateManager.aclRules[ruleKey];
        });
        templateManager.aclRules = aux;
        //Force framework to apply ACL Rules

        eventAggregator.publish('menuBuilded');
    };

    var replaceProductAttrOnMenu = function (productMenu) {
        attrToReplace.forEach(function (attr) {
            templateManager.project[attr] = productMenu[attr];
        });
    };

    var setLocalProductMenu = function () {
        if (!$window.modulesPackage) {
            requestProductMenu().then(function (productMenu) {
                localProductMenu = productMenu;
            });
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    if(!window.notUseMicroservice) {
        eventAggregator.subscribe('loginSuccess', function (loginResponse) {
            if (!$window.modulesPackage && !$window.keepTitle) {
                replaceProductAttrOnMenu(localProductMenu);
                templateManager.updateTitle(templateManager.project.label);
            }
            var filter = [{
                "name": "CONCATENATE_GROUP_MENU",
                "operator" : "=",
                "value" :  true
            }, {
                "name": "VISIBLE_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.VISIBLE_ELEMENTS || []
            }, {
                "name": "EXPIRED_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.EXPIRED_ELEMENTS || []
            }, {
                "name": "URL_ORIGIN",
                "operator" : "=",
                "value" :  window.location.origin
            }, {
                "name": "CURRENT_MODULE",
                "operator" : "=",
                "value" :  getCurrentModule()
            }, {
                "name": "CURRENT_LANGUAGE",
                "operator" : "=",
                "value" :  window.lang
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_buildMenu', filter)
                .then(prepareMenus)
                .catch(ApplicationContext.LibUtilities.showErrorMessage);
        });

        setLocalProductMenu();
    }
};

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuBuilder', MenuBuilder);
});